package jesse.keeblarcraft.AttributeMgr.AttributeNodes.FactionNodes;

import java.util.HashMap;
import java.util.List;

import org.apache.logging.log4j.core.jmx.Server;

import jesse.keeblarcraft.AttributeMgr.AttributeNodes.AbstractNode;
import jesse.keeblarcraft.FactionMgr.Callbacks.PlayerCommandFlightCallback;
import jesse.keeblarcraft.FactionMgr.Callbacks.PlayerEnteredBaseCallback;
import jesse.keeblarcraft.FactionMgr.Callbacks.PlayerExitedBaseCallback;
import net.minecraft.entity.player.PlayerAbilities;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.text.Text;
import net.minecraft.util.ActionResult;

public class FactionFlight extends AbstractNode {
    private final int flightSpeed = 1;
    private ServerPlayerEntity player;
    private float SPEED_SCALAR = 40.0f; // The value to scale this correctly to CREATIVE flight is '20.0f' however faction flight is slower than creative intentionally
    private Boolean canFly = false;

    @Override
    public String GetNodeTitle() {
        return "faction_flight";
    }

    @Override
    public String GetNodeDescription() {
        return "This is a temporary-unlock value when you are around your factions home base block and it has flight unlocked for members!";
    }

    @Override
    public HashMap<String, List<String>> GetDetails() {
        HashMap<String, List<String>> ret = new HashMap<String, List<String>>();
        ret.put("Faction Flight", List.of("Grants flight when near faction base if faction has it unlocked"));
        return ret;
    }

    public Boolean ToggleFlight(ServerPlayerEntity player) {
        // Only bother with flight if the player can fly
        Boolean isFlying = false;
        if (canFly) {
            PlayerAbilities abilities = player.getAbilities();

            // Flight toggle off
            if (abilities.flying) {
                // Disable flight
                abilities.flying = false;
                abilities.allowFlying = false;
                abilities.setFlySpeed(0);
            } else {
                // Flight toggle on
                isFlying = true;
                abilities.allowFlying = true;
                abilities.setFlySpeed((float) (flightSpeed / SPEED_SCALAR)); // Dividing by 20f yields max clamp value of 0.5 since MC does 0.0-> 1.0 flight. 0.1 flight is too fast!
            }
        }
        player.sendAbilitiesUpdate();
        return isFlying;
    }

    public void ResetPlayer(ServerPlayerEntity player) {
        PlayerAbilities abilities = player.getAbilities();
        if (abilities.flying) {
            // Disable flight
            abilities.flying = false;
            abilities.allowFlying = false;
            abilities.setFlySpeed(0);
        }
        player.sendAbilitiesUpdate();
    }

    @Override
    public void RegisterCallbacks(ServerPlayerEntity playerObj, ServerWorld worldObj) {
        // PlayerInBaseCallback.EVENT.register((player, world) -> {
        //     player.sendMessage(Text.of("Welcome home! Feel free to fly around!"));
        //     return ActionResult.SUCCESS;
        // });

        PlayerEnteredBaseCallback.EVENT.register((player, world) -> {
            player.sendMessage(Text.of("Faction flight enabled"));
            canFly = true;
            ServerPlayerEntity serverPlayer = (ServerPlayerEntity) player;
            ToggleFlight(serverPlayer);
            return ActionResult.SUCCESS;
        });

        PlayerCommandFlightCallback.EVENT.register((player, world) -> {
            ServerPlayerEntity serverPlayer = (ServerPlayerEntity) player;
            Boolean isFlying = ToggleFlight(serverPlayer);
            if(isFlying) {
                serverPlayer.sendMessage(Text.of("Flight turned on"));
            } else if (!isFlying && canFly) {
                // Infers toggled off
                serverPlayer.sendMessage(Text.of("Flight turned off"));
            } else {
                // Means player is not in fly-zone
                serverPlayer.sendMessage(Text.of("You can only fly within the bounds of your faction base!"));
            }
            return ActionResult.SUCCESS;
        });

        PlayerExitedBaseCallback.EVENT.register((player, world) -> {
            player.sendMessage(Text.of("Faction flight disabled"));
            canFly = false;
            ServerPlayerEntity serverPlayer = (ServerPlayerEntity) player;
            ResetPlayer(serverPlayer);
            return ActionResult.SUCCESS;
        });
    }

}
